@@ -194,6 +194,7 @@ class Statement(QObject):   # derived from QObject to have proper string transla
                 continue
             asset = self._find_in_list(self._data[FOF.ASSETS], "id", symbol['asset'])
             search_data = {'symbol': symbol['symbol'], 'type': self._asset_types[asset['type']]}
+            self._uppend_keys_from(search_data, asset, ['isin'])
             data = self._find_in_list(self._data[FOF.ASSETS_DATA], "asset", symbol['asset'])
             if data is not None:
                 self._uppend_keys_from(search_data, data, ['expiry'])
@@ -470,7 +471,7 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             if len(filtered) == 1:
                 return filtered[0]
             else:
-                raise Statement_ImportError(self.tr("Multiple match for ") + f"'{key}'='{value}'")
+                raise Statement_ImportError(self.tr("Multiple match for ") + f"'{key}'='{value}': {filtered}")
 
     # Method finds currency in current statement data. New currency is created if no currency was found.
     # Returns currency id
