@@ -1,109 +1,14 @@
-from PySide6.QtCore import Qt, QAbstractItemModel, QModelIndex
-from PySide6.QtSql import QSqlTableModel, QSqlRelationalTableModel, QSqlRelation, QSqlRelationalDelegate
-from PySide6.QtWidgets import QHeaderView
-from jal.db.helpers import db_connection, executeSQL, readSQL
+from PySide6.QtCore import Qt, QModelIndex
+from PySide6.QtSql import QSqlRelation, QSqlRelationalDelegate, QSqlIndex
+from PySide6.QtWidgets import QToolBar, QAbstractItemView
+from jal.db.helpers import readSQL
+from jal.db.reference_models import AbstractReferenceListModel, SqlTreeModel
 from jal.widgets.delegates import TimestampDelegate, BoolDelegate, FloatDelegate, \
     PeerSelectorDelegate, AssetSelectorDelegate
 from jal.widgets.reference_data import ReferenceDataDialog
+from jal.widgets.asset_dialog import AssetDialog
 from jal.widgets.delegates import GridLinesDelegate
-
-
-# ----------------------------------------------------------------------------------------------------------------------
-class AbstractReferenceListModel(QSqlRelationalTableModel):
-    @property
-    def completion_model(self):
-        return self._completion_model
-
-    def __init__(self, table, parent_view):
-        self._view = parent_view
-        self._table = table
-        self._columns = []
-        self._default_name = "name"
-        self._group_by = None
-        self._sort_by = None
-        self._hidden = []
-        self._stretch = None
-        self._default_values = {}   # To fill in default values for fields allowed to be NULL
-        QSqlRelationalTableModel.__init__(self, parent=parent_view, db=db_connection())
-        self.setJoinMode(QSqlRelationalTableModel.LeftJoin)
-        self.setTable(self._table)
-        self.setEditStrategy(QSqlTableModel.OnManualSubmit)
-        self.select()
-        # This is auxiliary 'plain' model of the same table - to be given as QCompleter source of data
-        self._completion_model = QSqlTableModel(parent=parent_view, db=db_connection())
-        self._completion_model.setTable(self._table)
-        self._completion_model.select()
-
-    @property
-    def group_by(self):
-        return self._group_by
-
-    def fieldIndex(self, field):
-        column_data = [i for i, column in enumerate(self._columns) if column[0] == field]
-        if len(column_data) > 0:
-            return column_data[0]
-        else:
-            return -1
-
-    def headerData(self, section, orientation=Qt.Horizontal, role=Qt.DisplayRole):
-        if orientation == Qt.Horizontal:
-            if role == Qt.DisplayRole:
-                return self._columns[section][1]
-        return None
-
-    def configureView(self):
-        self.setSorting()
-        self.hideColumns()
-        self.setStretching()
-        font = self._view.horizontalHeader().font()
-        font.setBold(True)
-        self._view.horizontalHeader().setFont(font)
-
-    def setSorting(self):
-        if self._sort_by:
-            self.setSort(self.fieldIndex(self._sort_by), Qt.AscendingOrder)
-
-    def hideColumns(self):
-        for column_name in self._hidden:
-            self._view.setColumnHidden(self.fieldIndex(column_name), True)
-
-    def setStretching(self):
-        if self._stretch:
-            self._view.horizontalHeader().setSectionResizeMode(self.fieldIndex(self._stretch), QHeaderView.Stretch)
-
-    def getId(self, index):
-        return self.record(index.row()).value('id')
-
-    def getName(self, index):
-        return self.getFieldValue(self.getId(index), self._default_name)
-
-    def getFieldValue(self, item_id, field_name):
-        return readSQL(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
-
-    def addElement(self, index, in_group=0):
-        row = index.row() if index.isValid() else 0
-        assert self.insertRows(row, 1)
-        new_record = self.record()
-        if in_group != 0:
-            new_record.setValue(self.fieldIndex(self._group_by), in_group)   # by index as it is lookup field
-        for field in self._default_values:
-            new_record.setValue(self.fieldIndex(field), self._default_values[field])
-        self.setRecord(row, new_record)
-
-    def removeElement(self, index):
-        if index.isValid():
-            row = index.row()
-        else:
-            return
-        assert self.removeRow(row)
-
-    def locateItem(self, item_id, use_filter=''):
-        raise NotImplementedError("locateItem() method is not defined in subclass of AbstractReferenceListModel")
-
-    def updateItemType(self, index, new_type):
-        id = self.getId(index)
-        executeSQL(f"UPDATE {self._table} SET {self._group_by}=:new_type WHERE id=:id",
-                   [(":new_type", new_type), (":id", id)])
+from jal.net.downloader import QuoteDownloader
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -155,7 +60,7 @@ class AccountListModel(AbstractReferenceListModel):
         self._bool_delegate = None
         self._default_values = {'active': 1, 'reconciled_on': 0, 'country_id': 0}
         self.setRelation(self.fieldIndex("type_id"), QSqlRelation("account_types", "id", "name"))
-        self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "name"))
+        self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
         self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "code"))
 
     def configureView(self):
@@ -181,7 +86,8 @@ class AccountListModel(AbstractReferenceListModel):
         return type_id
 
     def locateItem(self, item_id, use_filter=''):
-        row = readSQL(f"SELECT row_number FROM (SELECT ROW_NUMBER() OVER (ORDER BY name) AS row_number, id "
+        row = readSQL(f"SELECT row_number FROM ("
+                      f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
                       f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
         if row is None:
             return QModelIndex()
@@ -230,32 +136,35 @@ class AccountListDialog(ReferenceDataDialog):
 class AssetListModel(AbstractReferenceListModel):
     def __init__(self, table, parent_view):
         AbstractReferenceListModel.__init__(self, table, parent_view)
+        pk = QSqlIndex()   # Manual primary key setup is required as we use underlying sql view instead of sql table
+        pk.append(self.record().field("id"))
+        self.setPrimaryKey(pk)
         self._columns = [("id", ''),
-                         ("name", self.tr("Symbol")),
                          ("type_id", ''),
+                         ("symbol", self.tr("Symbol")),
                          ("full_name", self.tr("Name")),
                          ("isin", self.tr("ISIN")),
+                         ("currency_id", self.tr("Currency")),
                          ("country_id", self.tr("Country")),
-                         ("src_id", self.tr("Data source")),
-                         ("expiry", self.tr("Expiry"))]
-        self._sort_by = "name"
+                         ("quote_source", self.tr("Data source"))]
+        self._default_name = "symbol"
+        self._sort_by = "symbol"
         self._group_by = "type_id"
         self._hidden = ["id", "type_id"]
         self._stretch = "full_name"
         self._lookup_delegate = None
         self._timestamp_delegate = None
-        self._default_values = {'isin': '', 'country_id': 0, 'src_id': -1, 'expiry': 0}
+        self._default_values = {'isin': '', 'country_id': 0, 'quote_source': -1}
         self.setRelation(self.fieldIndex("type_id"), QSqlRelation("asset_types", "id", "name"))
+        self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
         self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "name"))
-        self.setRelation(self.fieldIndex("src_id"), QSqlRelation("data_sources", "id", "name"))
+        self.setRelation(self.fieldIndex("quote_source"), QSqlRelation("data_sources", "id", "name"))
 
     def configureView(self):
         super().configureView()
         self._lookup_delegate = QSqlRelationalDelegate(self._view)
         self._view.setItemDelegateForColumn(self.fieldIndex("country_id"), self._lookup_delegate)
-        self._view.setItemDelegateForColumn(self.fieldIndex("src_id"), self._lookup_delegate)
-        self._timestamp_delegate = TimestampDelegate(parent=self._view, display_format='%d/%m/%Y')
-        self._view.setItemDelegateForColumn(self.fieldIndex("expiry"), self._timestamp_delegate)
+        self._view.setItemDelegateForColumn(self.fieldIndex("quote_source"), self._lookup_delegate)
 
     def getAssetType(self, item_id: int) -> int:
         type_id = readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
@@ -263,7 +172,8 @@ class AssetListModel(AbstractReferenceListModel):
         return type_id
 
     def locateItem(self, item_id, use_filter=''):
-        row = readSQL(f"SELECT row_number FROM (SELECT ROW_NUMBER() OVER (ORDER BY name) AS row_number, id "
+        row = readSQL(f"SELECT row_number FROM ("
+                      f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
                       f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
         if row is None:
             return QModelIndex()
@@ -273,7 +183,7 @@ class AssetListModel(AbstractReferenceListModel):
 class AssetListDialog(ReferenceDataDialog):
     def __init__(self):
         ReferenceDataDialog.__init__(self)
-        self.table = "assets"
+        self.table = "assets_ext"
         self.model = AssetListModel(self.table, self.DataView)
         self.DataView.setModel(self.model)
         self.model.configureView()
@@ -281,22 +191,32 @@ class AssetListDialog(ReferenceDataDialog):
         super()._init_completed()
 
     def setup_ui(self):
-        self.search_field = "assets.full_name"
+        self.search_field = "assets_ext.full_name"
         self.setWindowTitle(self.tr("Assets"))
         self.SearchFrame.setVisible(True)
         self.Toggle.setVisible(False)
 
+        self.custom_editor = True
+        self.DataView.setEditTriggers(QAbstractItemView.NoEditTriggers)
+
         self.GroupLbl.setVisible(True)
         self.GroupLbl.setText(self.tr("Asset type:"))
         self.GroupCombo.setVisible(True)
         self.group_key_field = self.model.group_by
         self.group_key_index = self.model.fieldIndex(self.model.group_by)
         self.group_fkey_field = "id"
+        self.filter_field = "currency_id"
         relation_model = self.model.relationModel(self.group_key_index)
         self.GroupCombo.setModel(relation_model)
         self.GroupCombo.setModelColumn(relation_model.fieldIndex("name"))
         self.group_id = relation_model.data(relation_model.index(0, relation_model.fieldIndex(self.group_fkey_field)))
 
+        self.toolbar = QToolBar(self)
+        self.search_layout.addWidget(self.toolbar)
+        action = self.toolbar.addAction(self.tr("Update data"))
+        action.setToolTip(self.tr("Update assets data from their exchanges"))
+        action.triggered.connect(self.updateExchangeData)
+
     def locateItem(self, item_id):
         type_id = self.model.getAssetType(item_id)
         if type_id == 0:
@@ -305,213 +225,12 @@ class AssetListDialog(ReferenceDataDialog):
         item_idx = self.model.locateItem(item_id, use_filter=self._filter_text)
         self.DataView.setCurrentIndex(item_idx)
 
+    def updateExchangeData(self):
+        QuoteDownloader().updataData()
 
-# ----------------------------------------------------------------------------------------------------------------------
-class SqlTreeModel(QAbstractItemModel):
-    ROOT_PID = 0
+    def customEditor(self):
+        return AssetDialog()
 
-    @property
-    def completion_model(self):
-        return self._completion_model
-
-    def __init__(self, table, parent_view):
-        super().__init__(parent_view)
-        self._table = table
-        self._view = parent_view
-        self._default_name = "name"
-        self._stretch = None
-        # This is auxiliary 'plain' model of the same table - to be given as QCompleter source of data
-        self._completion_model = QSqlTableModel(parent=parent_view, db=db_connection())
-        self._completion_model.setTable(self._table)
-        self._completion_model.select()
-
-    def index(self, row, column, parent=None):
-        if parent is None:
-            return QModelIndex()
-        if not parent.isValid():
-            parent_id = self.ROOT_PID
-        else:
-            parent_id = parent.internalId()
-        child_id = readSQL(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT 1 OFFSET :row_n",
-                           [(":pid", parent_id), (":row_n", row)])
-        if child_id:
-            return self.createIndex(row, column, id=child_id)
-        return QModelIndex()
-
-    def parent(self, index):
-        if not index.isValid():
-            return QModelIndex()
-        child_id = index.internalId()
-        parent_id = readSQL(f"SELECT pid FROM {self._table} WHERE id=:id", [(":id", child_id)])
-        if parent_id == self.ROOT_PID:
-            return QModelIndex()
-        row = readSQL(f"SELECT row_number FROM ("
-                      f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
-                      f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
-                      f"WHERE id=:id", [(":id", parent_id)])
-        return self.createIndex(row-1, 0, id=parent_id)
-
-    def rowCount(self, parent=None):
-        if not parent.isValid():
-            parent_id = self.ROOT_PID
-        else:
-            parent_id = parent.internalId()
-        count = readSQL(f"SELECT COUNT(id) FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
-        if count:
-            return int(count)
-        else:
-            return 0
-
-    def columnCount(self, parent=None):
-        return len(self._columns)
-
-    def flags(self, index):
-        if not index.isValid():
-            return Qt.NoItemFlags
-        return Qt.ItemIsEditable | super().flags(index)
-
-    def data(self, index, role=Qt.DisplayRole):
-        if not index.isValid():
-            return None
-        item_id = index.internalId()
-        if role == Qt.DisplayRole:
-            col = index.column()
-            if (col >= 0) and (col < len(self._columns)):
-                return readSQL(f"SELECT {self._columns[col][0]} FROM {self._table} WHERE id=:id", [(":id", item_id)])
-            else:
-                return None
-        return None
-
-    def setData(self, index, value, role=Qt.EditRole):
-        if role != Qt.EditRole:
-            return False
-        if not index.isValid():
-            return False
-        item_id = index.internalId()
-        col = index.column()
-        db_connection().transaction()
-        _ = executeSQL(f"UPDATE {self._table} SET {self._columns[col][0]}=:value WHERE id=:id",
-                       [(":id", item_id), (":value", value)])
-        self.dataChanged.emit(index, index, Qt.DisplayRole | Qt.EditRole)
-        return True
-
-    def fieldIndex(self, field):
-        column_data = [i for i, column in enumerate(self._columns) if column[0] == field]
-        if len(column_data) > 0:
-            return column_data[0]
-        else:
-            return -1
-
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if orientation == Qt.Horizontal:
-            if role == Qt.DisplayRole:
-                return self._columns[section][1]
-        return None
-
-    def configureView(self):
-        self.setStretching()
-        font = self._view.header().font()
-        font.setBold(True)
-        self._view.header().setFont(font)
-
-    def setStretching(self):
-        if self._stretch:
-            self._view.header().setSectionResizeMode(self.fieldIndex(self._stretch), QHeaderView.Stretch)
-
-    def getId(self, index):
-        if not index.isValid():
-            return None
-        return index.internalId()
-
-    def getName(self, index):
-        item_id = self.getId(index)
-        if item_id is not None:
-            self.getFieldValue(item_id, self._default_name)
-
-    def getFieldValue(self, item_id, field_name):
-        return readSQL(f"SELECT {field_name} FROM {self._table} WHERE id=:id", [(":id", item_id)])
-
-    def deleteWithChilderen(self, parent_id: int) -> None:
-        query = executeSQL(f"SELECT id FROM {self._table} WHERE pid=:pid", [(":pid", parent_id)])
-        while query.next():
-            self.deleteWithChilderen(query.value(0))
-        _ = executeSQL(f"DELETE FROM {self._table} WHERE id=:id", [(":id", parent_id)])
-
-    def insertRows(self, row, count, parent=None):
-        if parent is None:
-            return False
-        if not parent.isValid():
-            parent_id = self.ROOT_PID
-        else:
-            parent_id = parent.internalId()
-
-        self.beginInsertRows(parent, row, row + count - 1)
-        db_connection().transaction()
-        _ = executeSQL(f"INSERT INTO {self._table}(pid, name) VALUES (:pid, '')", [(":pid", parent_id)])
-        self.endInsertRows()
-        self.layoutChanged.emit()
-        return True
-
-    def removeRows(self, row, count, parent=None):
-        if parent is None:
-            return False
-        if not parent.isValid():
-            parent_id = self.ROOT_PID
-        else:
-            parent_id = parent.internalId()
-
-        self.beginRemoveRows(parent, row, row + count - 1)
-        db_connection().transaction()
-        query = executeSQL(f"SELECT id FROM {self._table} WHERE pid=:pid LIMIT :row_c OFFSET :row_n",
-                           [(":pid", parent_id), (":row_c", count), (":row_n", row)])
-        while query.next():
-            self.deleteWithChilderen(query.value(0))
-        self.endRemoveRows()
-        self.layoutChanged.emit()
-        return True
-
-    def addElement(self, index, in_group=0):  # in_group is used for plain model only, not tree
-        row = index.row()
-        assert self.insertRows(row, 1, index.parent())
-
-    def addChildElement(self, index):
-        assert self.insertRows(0, 1, index)
-        self._view.expand(index)
-
-    def removeElement(self, index):
-        row = index.row()
-        assert self.removeRows(row, 1, index.parent())
-
-    def submitAll(self):
-        _ = executeSQL("COMMIT")
-        self.layoutChanged.emit()
-        return True
-
-    def revertAll(self):
-        _ = executeSQL("ROLLBACK")
-        self.layoutChanged.emit()
-
-    # expand all parent elements for tree element with given index
-    def expand_parent(self, index):
-        parent = index.parent()
-        if parent.isValid():
-            self.expand_parent(parent)
-        self._view.expand(index)
-
-    # find item by ID and make it selected in associated self._view
-    def locateItem(self, item_id):
-        row = readSQL(f"SELECT row_number FROM ("
-                      f"SELECT ROW_NUMBER() OVER (ORDER BY id) AS row_number, id, pid "
-                      f"FROM {self._table} WHERE pid IN (SELECT pid FROM {self._table} WHERE id=:id)) "
-                      f"WHERE id=:id", [(":id", item_id)])
-        if row is None:
-            return
-        item_idx = self.createIndex(row-1, 0, id=item_id)
-        self.expand_parent(item_idx)
-        self._view.setCurrentIndex(item_idx)
-
-    def setFilter(self, filter_str):
-        pass
 
 # ----------------------------------------------------------------------------------------------------------------------
 class PeerTreeModel(SqlTreeModel):
@@ -693,11 +412,16 @@ class QuotesListModel(AbstractReferenceListModel):
         self._columns = [("id", ''),
                          ("timestamp", self.tr("Date")),
                          ("asset_id", self.tr("Asset")),
+                         ("currency_id", self.tr("Currency")),
                          ("quote", self.tr("Quote"))]
         self._hidden = ["id"]
         self._default_name = "quote"
+        self._sort_by = "timestamp"
+        self._stretch = "asset_id"
         self._asset_delegate = None
         self._timestamp_delegate = None
+        self._lookup_delegate = None
+        self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
 
     def configureView(self):
         super().configureView()
@@ -709,6 +433,8 @@ class QuotesListModel(AbstractReferenceListModel):
         self._view.setItemDelegateForColumn(self.fieldIndex("asset_id"), self._asset_delegate)
         self._timestamp_delegate = TimestampDelegate(parent=self._view)
         self._view.setItemDelegateForColumn(self.fieldIndex("timestamp"), self._timestamp_delegate)
+        self._lookup_delegate = QSqlRelationalDelegate(self._view)
+        self._view.setItemDelegateForColumn(self.fieldIndex("currency_id"), self._lookup_delegate)
 
 
 class QuotesListDialog(ReferenceDataDialog):
@@ -722,7 +448,7 @@ class QuotesListDialog(ReferenceDataDialog):
         super()._init_completed()
 
     def setup_ui(self):
-        self.search_field = "asset_id-assets-id-name"
+        self.search_field = "asset_id-assets_ext-id-symbol"
         self.SearchFrame.setVisible(True)
         self.setWindowTitle(self.tr("Quotes"))
         self.Toggle.setVisible(False)
