@@ -6,7 +6,7 @@ from PySide6.QtWidgets import QLabel, QDateTimeEdit, QDateEdit, QLineEdit
 from jal.widgets.abstract_operation_details import AbstractOperationDetails
 from jal.widgets.reference_selector import AccountSelector, AssetSelector
 from jal.widgets.delegates import WidgetMapperDelegateBase
-from jal.constants import TransactionType
+from jal.db.operations import LedgerTransaction
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -15,6 +15,7 @@ class TradeWidgetDelegate(WidgetMapperDelegateBase):
         WidgetMapperDelegateBase.__init__(self, parent)
         self.delegates = {'timestamp': self.timestamp_delegate,
                           'settlement': self.timestamp_delegate,
+                          'asset_id': self.symbol_delegate,
                           'qty': self.float_delegate,
                           'price': self.float_delegate,
                           'fee': self.float_delegate}
@@ -25,7 +26,7 @@ class TradeWidget(AbstractOperationDetails):
     def __init__(self, parent=None):
         AbstractOperationDetails.__init__(self, parent)
         self.name = "Trade"
-        self.operation_type = TransactionType.Trade
+        self.operation_type = LedgerTransaction.Trade
 
         self.date_label = QLabel(self)
         self.settlement_label = QLabel()
